from dataclasses import dataclass


@dataclass
class Schedule:
    name: str = ""
    link: str = ""

    def to_json(self) -> str:
        return f"({self.name},{self.link})"

    def from_json(self, data: str):
        self.name: str = data[1:data.find(",")]
        self.link: str = data[data.find(",") + 1:-1]
        
        return self




import sys
import time
from typing import List
from .cache import Cache
from .schedule import Schedule
from .data_fetcher import DataFetcher, ScheduleData


class Scheduler:
    USE_WHITELIST: bool = False 
    COMMAND_NAME: str = "liusch"


    def __init__(self) -> None:
        self.cache: Cache = Cache()
        self.data_fetcher: DataFetcher = DataFetcher()
            
    def get_next_entry(self) -> str:
        """Get the next calendar entry"""
        data: List[ScheduleData] = self.data_fetcher.get_data(self.cache.get_schedules())

        entry: ScheduleData | None = self._get_next_entry(data)

        if entry is None:
            return ""

        course_code: str = entry.course_codes[0]

        if self.USE_WHITELIST:
            course_code: str = self._get_whitelisted_course(entry.course_codes)

        if not course_code:
            return ""

        return f"{entry.start_date} | {entry.start_time}-{entry.end_time} | {course_code} | {entry.course_type} | {entry.locations[0]}"

    def parse_args(self, argv: List[str]) -> None:
        # Exit if there are no arguments
        if len(argv) <= 1:
            print(self.get_next_entry())
            return

        match argv[1]:
            case "--add-schedule" | "-a":
                self._add_schedule(argv[2:])
            case "--add-course" | "-aw":
                self._add_courses(argv[2:])
            case "--remove-schedule" | "-rm":
                self._remove_schedule(argv[2:])
            case "--remove-course" | "-rmw":
                self._remove_courses(argv[2:])
            case "--list-schedules" | "-ls":
                self._list_schedules()
            case "--list-courses" | "-lsw":
                print(self.cache.get_courses())
            case "--help" | "-h":
                self._display_help()
            case _:
                print(f"{argv[1]} is not a valid argument. Try -h for help.")

    def _add_schedule(self, schedules: List[str]) -> None:
        if not len(schedules) % 2 == 0:
            print("Names and links do not match.")
            return

        for i in range(0, len(schedules), 2):
            name: str = schedules[i]
            link: str = schedules[i + 1]

            self.cache.add_schedule(name, link)

    def _add_courses(self, courses: List[str]) -> None:
        for course in courses:
            self.cache.add_course(course)

    def _remove_schedule(self, schedules: List[str]) -> None:
        for schedule in schedules:
            name: str = schedule

            self.cache.remove_schedule(name)

    def _remove_courses(self, courses: List[str]) -> None:
        for course in courses:
            self.cache.remove_course(course)

    def _list_schedules(self) -> None:
        for schedule in self.cache.get_schedules():
            print("{:<24} {}".format(schedule.name, schedule.link))

    def _get_whitelisted_course(self, courses: List[str]) -> str:
        """Get the course that's in the whitelist from a list of courses"""
        whitelist: List[str] = self.cache.get_courses()

        for course in courses:
            print(f"{course} in {whitelist}")
            if course in whitelist:
                return course

        return ""

    def _get_next_entry(self, data: List[ScheduleData]) -> ScheduleData | None:
        """Get the next relevant entry"""
        for schedule in data:
            if schedule.end_unix >= time.time():
                return schedule

        return None
    
    def _display_help(self) -> None:
        self._print_bold("USAGE")
        print(f"    {self.COMMAND_NAME} [-a 'name' 'link' ...] [-aw 'course' ...] [-rm 'name' ...] [-rmw 'course' ...] [-ls] [-lsw] [-h]")

        self._print_bold("\nOPTIONS")

        self._print_bold("   -a, --add-schedule 'name' 'link'")
        print("        Add a new schedule with a name and a TimeEdit link.\n")

        self._print_bold("   -aw, --add-course 'course'")
        print("        Add a course code to the whitelist. If a schedule entry has several codes \n        the whitelisted one will be displayed.\n")

        self._print_bold("   -rm, --remove-schedule 'name'")
        print("        Remove a schedule entry given it's name.\n")

        self._print_bold("   -rmw, --remove-course 'course'")
        print("        Remove a whitelisted course given it's course code.\n")

        self._print_bold("   -ls, --list-schedules")
        print("        Display all loaded shedules.\n")

        self._print_bold("   -lsw, --list-courses")
        print("        Display all whitelisted course codes.\n")

        self._print_bold("   -h, --help")
        print("        Display this help message.")

    def _print_bold(self, text: str) -> None:
        print(f"\033[1m{text}\033[0m")


def main() -> None:
    scheduler: Scheduler = Scheduler()
    scheduler.parse_args(sys.argv)


if __name__ == "__main__":
    main()




from __future__ import annotations
import os
import json
from typing import List
from .schedule import Schedule

    
class Cache:
    DIR_NAME: str = "python-liu-schedule"
    SCHEDULES_FILE: str = "schedules.json"
    COURSES_FILE: str = "course_whitelist.json"


    def __init__(self) -> None:
        self._dir: str = self._init_cache_dir()
        self._schedules: str = os.path.join(self._dir, self.SCHEDULES_FILE)
        self._courses: str = os.path.join(self._dir, self.COURSES_FILE)


    def add_schedule(self, name: str, link: str) -> None:
        data: List[Schedule] = self._read_schedules()
        data.append(Schedule(name, link))
        self._write_schedules(data)

    def add_course(self, course: str) -> None:
        """Add a course to the whitelist"""
        data: List[str] = self._read_courses()

        if course not in data:
            data.append(course)

        self._write_courses(data)

    def remove_schedule(self, name: str) -> None:
        schedules: List[Schedule] = [s for s in self._read_schedules() if not s.name == name]
        self._write_schedules(schedules)

    def remove_course(self, name: str) -> None:
        courses: List[str] = [c for c in self._read_courses() if not c == name]
        self._write_courses(courses)

    def get_schedules(self) -> List[Schedule]:
        return self._read_schedules()

    def get_courses(self) -> List[str]:
        return self._read_courses()

    def _read_schedules(self) -> List[Schedule]:
        return [Schedule().from_json(schedule) for schedule in self._read_json(self._schedules)]

    def _read_courses(self) -> List[str]:
        return self._read_json(self._courses)

    def _read_json(self, file_path: str) -> List[str]:
        try:
            with open(file_path, "r") as file:
                return json.load(file)
        except (FileNotFoundError, json.JSONDecodeError):
            return []

    def _write_schedules(self, schedule_links: List[Schedule]) -> None:
        with open(self._schedules, "w") as file:
            json.dump(schedule_links, file, default=lambda o: o.to_json())

    def _write_courses(self, courses: List[str]) -> None:
        with open(self._courses, "w") as file:
            json.dump(courses, file)

    def _init_cache_dir(self) -> str:
        # Get the home catalogue path
        home: str | None = os.getenv("HOME", os.getenv("USERPROFILE"))

        if home is None:
            raise FileNotFoundError("Home catalogue could not be found.")

        # Get the cache directory path
        cache_dir: str = os.getenv("XDG_CACHE_HOME", os.path.join(home, ".cache"))

        if cache_dir is None:
            raise FileNotFoundError("Cache directory could not be found.")

        cache_dir: str = os.path.join(cache_dir, self.DIR_NAME)
        
        # Create the directory if it does not exist
        os.makedirs(cache_dir, exist_ok=True)

        return cache_dir




import requests
import re
import calendar
import time
from re import Match
from typing import List
from requests import Response
from .schedule import Schedule
from datetime import date, datetime, timezone
from dataclasses import dataclass, field


@dataclass
class ScheduleData:
    start_date: str = ""
    start_time: str = ""
    start_unix: float = 0
    end_date: str = ""
    end_time: str = ""
    end_unix: float = 0
    course_codes: List[str] = field(default_factory=lambda: [])
    course_type: str = ""
    locations: List[str] = field(default_factory=lambda: [])


class DataFetcher:
    ENTRY_SEPARATOR: str = "BEGIN:VEVENT"
    COURSE_CODE_SEPARATOR: str = r"\, "
    DATA_ENCODING: str = "utf-8"

    DATETIME_FORMAT: str = "%Y%m%dT%H%M%SZ"
    WEEK_IN_SECONDS: int = 604800
    
    START_TIME_RE: str = "(?<=DTSTART:).*"
    END_TIME_RE: str = "(?<=DTEND:).*"
    COURSE_CODE_RE: str = r"(?<=SUMMARY:).*(?=U)" 
    TYPE_RE: str = r"(?<=Undervisningstyp: ).*?(?=\\)" 
    LOCATION_RE: str = r"(?<=Lokal: ).*?(?=\\n|\n|\r)"


    def get_data(self, schedules: List[Schedule]) -> List[ScheduleData]:
        """Get the sorted schedule data"""
        data: List[ScheduleData] = []

        for schedule in schedules:
            content: str = self._fetch_from_web(schedule.link)

            if content:
                data += self._parse_data(content)

        # Sort the entries
        data.sort(key=lambda o: o.start_unix)

        return data

    def _parse_data(self, content: str) -> List[ScheduleData]:
        """Parse the raw data into ScheduleData"""
        data: List[ScheduleData] = []

        for entry in content.split(self.ENTRY_SEPARATOR):
            schedule: ScheduleData = ScheduleData()

            start_datetime_text: str = self._match_regex(entry, self.START_TIME_RE).strip()
            end_datetime_text: str = self._match_regex(entry, self.END_TIME_RE).strip()

            # Skip if the times couldn't be parsed
            if not start_datetime_text or not end_datetime_text:
                continue

            # Convert the dates to local datetimes
            start_datetime: datetime = self._get_datetime_from_text(start_datetime_text)
            end_datetime: datetime = self._get_datetime_from_text(end_datetime_text)

            # Get the local dates
            schedule.start_date = self._get_date_from_datetime(start_datetime)
            schedule.end_date = self._get_date_from_datetime(end_datetime)

            # Get the local time
            schedule.start_time = self._get_time_from_datetime(start_datetime)
            schedule.end_time = self._get_time_from_datetime(end_datetime)

            # Save unix time to be able to sort the schedule
            schedule.start_unix = start_datetime.timestamp()
            schedule.end_unix = end_datetime.timestamp()
            
            # Get the course code, type and location
            course_codes_text = self._match_regex(entry, self.COURSE_CODE_RE)
            schedule.course_codes = self._get_courses_from_text(course_codes_text)
            schedule.course_type = self._match_regex(entry, self.TYPE_RE)
            schedule.locations = self._match_all_regex(entry, self.LOCATION_RE)

            data.append(schedule)

        return data

    def _match_regex(self, text: str, regex: str) -> str:
        match: Match[str] | None = re.search(regex, text)

        return match.group() if match is not None else ""

    def _match_all_regex(self, text: str, regex: str) -> List[str]:
        return [entry.strip() for entry in re.findall(regex, text)]
    
    def _utc_to_local_time(self, utc_time: datetime) -> datetime:
        """Convert UTC time to local time"""
        return utc_time.replace(tzinfo=timezone.utc).astimezone(tz=None)

    def _get_datetime_from_text(self, text: str) -> datetime:
        date_time: datetime = datetime.strptime(text, self.DATETIME_FORMAT)
        return self._utc_to_local_time(date_time)
    
    def _get_date_from_datetime(self, date_time: datetime) -> str:
        """Get the formatted date from a datetime object"""
        # Return weekday if less than a week is left
        if date_time.timestamp() - time.time() < self.WEEK_IN_SECONDS:
            return calendar.day_name[date_time.weekday()]

        return date_time.strftime("%Y-%m-%d")

    def _get_time_from_datetime(self, date_time: datetime) -> str:
        """Get the formatted time from a datetime object"""
        return date_time.strftime("%H:%M")
    
    def _get_courses_from_text(self, text: str) -> List[str]:
        """Get the course types from a string"""
        return text.split(self.COURSE_CODE_SEPARATOR)[:-1] # Remove last empty element

    
    def _fetch_from_web(self, link: str):
        """Fetch schedule data from the web"""
        try:
            return requests.get(link).content.decode(self.DATA_ENCODING)
        except Exception:
            return ""




from setuptools import setup

setup(
    name="liusch",
    version="1.0",
    packages=["liusch"],
    entry_points={
        "console_scripts": [
            "liusch = liusch.main:main"
        ]
    },
)

